import factory

from ..models import Consumable


class ConsumableFactory(factory.django.DjangoModelFactory):
    """
    Factory to create instance of a Consumable.
    """
    title = factory.Sequence(lambda n: "Consumable {0}".format(n))
    slug = factory.Sequence(lambda n: "consumable-{0}".format(n))

    class Meta:
        model = Consumable



import factory

from ..models import Brand
from ..utils.imaging import DjangoSampleImageCrafter


class BrandFactory(factory.django.DjangoModelFactory):
    """
    Factory to create instance of a Brand.
    """
    title = factory.Sequence(lambda n: "Brand {0}".format(n))
    slug = factory.Sequence(lambda n: "brand-{0}".format(n))
    description = factory.Faker("text", max_nb_chars=50)

    class Meta:
        model = Brand

    @factory.lazy_attribute
    def cover(self):
        """
        Fill cover field with generated image.

        Returns:
            django.core.files.File: File object.
        """
        crafter = DjangoSampleImageCrafter()
        return crafter.create()



import io

from PIL import Image as PILimage
from PIL import ImageDraw

from django.core.files import File


class SampleImageCrafter:
    """
    Craft a basic sample image, either a bitmap or a SVG.

    Basically every supported format from PIL should work however this code only knows
    about JPEG, GIF, PNG and SVG.

    Keyword Arguments:
        font (Pil.ImageFont): A font object to use. This is only used for bitmap image.
            It is strongly recommended to use a TrueType font. Default is None, this
            will use the default PIL font. Note than text without a TrueType font will
            be badly positionned (almost centered but largely shifted).
    """
    def __init__(self, *args, **kwargs):
        self.font = None

        if "font" in kwargs:
            self.font = kwargs.pop("font")

    def get_text_content(self, text, width, height):
        """
        Get the text content to include in image.

        Arguments:
            text (string or boolean): Either a string to use it as content or ``True``
                to make automatic content from given sizes such as ``320x240`` (for
                width value ``320`` and height value ``240``). If string is given it
                should be a short text else it is not guaranteed to fit. If you don't
                want text content, just pass a empty string.
            width (integer): Width value to display in automatic text content.
            height (integer): Height value to display in automatic text content.

        Returns:
            string: Content to include in image.
        """
        text_content = ""

        # Format default text
        if text is True:
            # width x height
            text_content = "{}x{}".format(width, height)
        # Or just use possible given custom string
        elif isinstance(text, str):
            text_content = text

        return text_content

    def get_file_extension(self, format_name):
        """
        Get correct file extension depending format name.

        Arguments:
            format_name (string): The format name to use to get the right file
                extension. It could be any format supported. ``JPEG`` and ``JPG`` will
                traduces to ``jpg`` file extension.

        Returns:
            string: File extension without leading dot.
        """
        file_extension = format_name.lower()

        # "jpeg" is an allowed alias for "jpg"
        if file_extension == "jpeg":
            file_extension = "jpg"

        return file_extension

    def get_mode(self, format_name):
        """
        Get correct image color mode depending format name.

        Arguments:
            format_name (string): The format name to use to get the right file
                extension. It could be any format supported. ``PNG`` will be a ``RGBA``
                mode, every other format will be ``RGB``.

        Returns:
            string: Image color mode name.
        """
        if format_name == "PNG":
            return "RGBA"

        return "RGB"

    def get_filename(self, file_extension, filename=None, bg_color=None):
        """
        Get filename.

        Arguments:
            file_extension (string): File extension to use in automatic filename. Can
                be anything if ``filename`` argument is given since it will be ignored.

        Keyword Arguments:
            filename (string): Custom filename to use, every other arguments won't be
                used to compute filename.
            bg_color (string): A color name to use in automatic filename. For real, this
                can be anything since it is not validated.

        Returns:
            string: File name.
        """
        if not filename:
            return "{}.{}".format(bg_color, file_extension)

        return filename

    def build(self, filename=None, size=(100, 100), bg_color="blue", text_color="white",
              text=None, format_name="PNG"):
        """
        Build config for content creation

        Keyword Arguments:
            filename (string): Custom file name (with file extension) to override the
                automatic file name (based on other given arguments). Default is
                ``None`` which will produce an automatic file name.
            size (tuple): A tuple of two integers respectively for width and height.
                Default to ``(100, 100)`` which will produce a square of 100 pixels.
            bg_color (string): Color name to use to paint image background. Default to
                ``blue``.
            text_color (string): Color name to use to draw possible content text.
                Default to ``white``.
            text (string): Custom content text to include in image. Default to ``None``
                which will produce an automatic content based on size. Use an empty
                string to avoid content text in image.
            format_name (string): Image format name to use. Default to ``PNG``.

        Returns:
            dict: Configuration to use to create image file.
        """
        # Split given size
        width, height = size

        config = {
            "size": size,
            "width": width,
            "height": height,
            "format_name": format_name,
            "mode": self.get_mode(format_name),
            "file_extension": self.get_file_extension(format_name),
            "bg_color": bg_color,
            "text": text,
            "text_color": text_color,
            "text_content": self.get_text_content(text, width, height),
        }

        config["filename"] = self.get_filename(
            config["file_extension"],
            filename=filename,
            bg_color=config["bg_color"],
        )

        return config

    def create_vectorial(self, width, height, bg_color, text_content=None,
                         text_color=None):
        """
        Create SVG content.

        Arguments:
            width (integer): Image width.
            height (integer): Image height.
            bg_color (object): Color name to paint image background.

        Keyword Arguments:
            text_content (string): Content text to include instead of automatic content
                text.
            text_color (string): Color name to draw text instead of default one. It is
                required if you want to use the custom context text.

        Returns:
            io.StringIO: SVG content in a string buffer.
        """
        svg = (
            '<svg xmlns="http://www.w3.org/2000/svg" '
            'role="img" aria-label="Placeholder" '
            'preserveAspectRatio="xMidYMid slice" focusable="false" '
            'viewBox="0 0 {width} {height}" style="text-anchor: middle">'
            '<rect width="100%" height="100%" fill="{bg_color}"></rect>'
        ).format(
            width=str(width),
            height=str(height),
            bg_color=bg_color,
        )

        if text_color and text_content:
            svg += (
                '<text x="50%" y="50%" fill="{text_color}" dy=".3em">{text}</text>'
            ).format(
                text=text_content,
                text_color=text_color,
            )

        svg += '</svg>'

        return io.StringIO(svg)

    def create_bitmap(self, mode, format_name, width, height, bg_color,
                      text_content=None, text_color=None):
        """
        Create Bitmap image object.

        Arguments:
            mode (string): Image color mode to use.
            format_name (string): Image format name to use.
            width (integer): Image width.
            height (integer): Image height.
            bg_color (object): Color name to paint image background.

        Keyword Arguments:
            text_content (string): Content text to include instead of automatic content
                text.
            text_color (string): Color name to draw text instead of default one. It is
                required if you want to use the custom context text.

        Returns:
            io.BytesIO: Image object in a byte buffer.
        """
        img = PILimage.new(mode, (width, height), bg_color)

        # Optional text, always centered
        if text_color and text_content:
            draw = ImageDraw.Draw(img)
            draw.text(
                (width / 2, height / 2),
                text_content,
                fill=text_color,
                font=self.font,
                anchor="mm",
            )

        output = io.BytesIO()
        img.save(output, format=format_name)

        return output

    def create(self, filename=None, size=(100, 100), bg_color="blue",
               text_color="white", text=None, format_name="PNG"):
        """
        Create an image inside a file object.

        Return a File object with a dummy generated image on the fly by PIL or
        possibly a SVG file.

        With default argument values the generated image will be a simple blue
        square in PNG with no text.

        Optionally, you can have any other supported format (JPEG, GIF, SVG), a custom
        background color and a text.

        Keyword Arguments:
            filename (string): Filename for created file, default to ``bg_color``
                value joined to extension with ``format`` value in lowercase (or
                ``jpg`` if format is ``JPEG``).
                Note than final filename may be different if all tests use the same
                one since Django will append a hash for uniqueness.
            format_name (string): Format name as available from PIL: ``JPEG``,
                ``PNG`` or ``GIF``. ``SVG`` format is also possible to create a
                dummy SVG file.
            size (tuple): A tuple of two integers respectively for width and height.
                Default to ``(100, 100)`` which will produce a square of 100 pixels.
            bg_color (string): Color value to fill image, this should be a valid value
                for ``PIL.ImageColor``:
                https://pillow.readthedocs.io/en/stable/reference/ImageColor.html
                or a valid HTML color name for SVG format. Default to "blue". WARNING:
                If you don't use named color (like "white" or "yellow"), you should
                give a custom filename to ``filename`` argument else the filename may
                be weird (like ``#111111.png``).
            text_color (string): Color value for text. This should be a valid value
                for ``PIL.ImageColor``:
                https://pillow.readthedocs.io/en/stable/reference/ImageColor.html
                Default to "white".
            text (string or boolean): ``True`` for automatic image size like
                ``320x240`` (for width value ``320`` and height value ``240``). ``None``
                or ``False`` to disable text drawing (this is the default value). A
                string for custom text, this should be a short text else it is not
                guaranteed to fit.

        Returns:
            object: File object.
        """
        config = self.build(
            filename=filename,
            size=size,
            bg_color=bg_color,
            text_color=text_color,
            text=text,
            format_name=format_name
        )

        if config["format_name"] == "SVG":
            output = self.create_vectorial(
                config["width"],
                config["height"],
                config["bg_color"],
                text_content=config["text_content"],
                text_color=config["text_color"],
            )
        else:
            output = self.create_bitmap(
                config["mode"],
                config["format_name"],
                config["width"],
                config["height"],
                config["bg_color"],
                text_content=config["text_content"],
                text_color=config["text_color"],
            )

        return output


class DjangoSampleImageCrafter(SampleImageCrafter):
    """
    Alike SampleImageCrafter but return a Django File instead of a file object.
    """
    def create(self, *args, **kwargs):
        """
        Create an image inside a Django File object.

        Arguments:
            *args: The same positional arguments as from SampleImageCrafter.
            **kwargs: The same keyword arguments as from SampleImageCrafter.

        Returns:
            django.core.files.File: File object.
        """
        config = self.build(*args, **kwargs)

        if config["format_name"] == "SVG":
            output = self.create_vectorial(
                config["width"],
                config["height"],
                config["bg_color"],
                text_content=config["text_content"],
                text_color=config["text_color"],
            )
        else:
            output = self.create_bitmap(
                config["mode"],
                config["format_name"],
                config["width"],
                config["height"],
                config["bg_color"],
                text_content=config["text_content"],
                text_color=config["text_color"],
            )

        return File(output, name=config["filename"])



from django.utils.translation import gettext_lazy as _
from django.urls import reverse
from haystack.query import RelatedSearchQuerySet
from haystack.generic_views import SearchView

from ..forms import GlobalSearchForm
from .mixins import AtoumBreadcrumMixin


class GlobalSearchView(AtoumBreadcrumMixin, SearchView):
    """
    View to implement search for indexed Atoum models.
    """
    template_name = "atoum/search/results.html"
    form_class = GlobalSearchForm
    crumb_title = _("Search")
    crumb_urlname = "atoum:search-results"

    @property
    def crumbs(self):
        return [
            (self.crumb_title, reverse(self.crumb_urlname)),
        ]

    def get_queryset(self):
        """
        Use a ``RelatedSearchQuerySet`` so we can use ``select_related()`` when loading
        model objects from search results.
        """
        if self.queryset is None:
            self.queryset = RelatedSearchQuerySet()
        return self.queryset



from django.conf import settings

try:
    from view_breadcrumbs import BaseBreadcrumbMixin
except ImportError:
    class BaseBreadcrumbMixin:
        """
        A dummy and empty mixin to use when 'django-view-breadcrumbs' is not available.
        """
        pass


class AtoumBreadcrumMixin(BaseBreadcrumbMixin):
    """
    A mixin to include base breadcrumb mixin if installed and some common
    breadcrumb options.
    """
    add_home = settings.ATOUM_BREADCRUMBS_SHOW_HOME



import factory

from ..models import Category
from .assortment import AssortmentFactory


class CategoryFactory(factory.django.DjangoModelFactory):
    """
    Factory to create instance of a Category.
    """
    assortment = factory.SubFactory(AssortmentFactory)
    title = factory.Sequence(lambda n: "Category {0}".format(n))
    slug = factory.Sequence(lambda n: "category-{0}".format(n))

    class Meta:
        model = Category



from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from smart_media.admin import SmartModelAdmin

from ..models import Brand


@admin.register(Brand)
class BrandAdmin(SmartModelAdmin):
    readonly_fields = ["created", "modified"]
    prepopulated_fields = {
        "slug": ("title",),
    }
    ordering = Brand.COMMON_ORDER_BY
    search_fields = [
        "title",
        "description",
    ]
    list_display = (
        "title",
        "count_products",
        "modified",
    )

    def count_products(self, obj):
        """
        Count related products.
        """
        return obj.product_set.count()
    count_products.short_description = _("Products")



from django.views.generic import TemplateView
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from ..models import Assortment, Category, Consumable, Product
from .mixins import AtoumBreadcrumMixin


class DashboardView(AtoumBreadcrumMixin, TemplateView):
    """
    Catalog dashboard view

    .. todo::
        This should list:

        * Catalog models
        * open shopping lists
        * stock resume

    """
    template_name = "atoum/dashboard.html"
    crumb_title = _("Catalog dashboard")
    crumb_urlname = "atoum:dashboard"

    @property
    def crumbs(self):
        return [
            (self.crumb_title, reverse(self.crumb_urlname)),
        ]

    def get_consumable_queryset(self):
        """
        Build consumable queryset.
        """
        qs = Consumable.objects.all().order_by(*Consumable.COMMON_ORDER_BY)

        return qs

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        context.update({
            "consumable_count": Consumable.objects.all().count(),
            "assortment_count": Assortment.objects.all().count(),
            "category_count": Category.objects.all().count(),
            "product_count": Product.objects.all().count(),
        })

        return context



from django.db import models
from django.utils.translation import gettext_lazy as _
from django.urls import reverse
from django.utils import timezone
from django.utils.html import format_html


class Assortment(models.Model):
    """
    Assortment of consumables.

    Attributes:
        consumable (models.ForeignKey): Required Consumable object.
        created (models.DateTimeField): Required creation datetime, automatically
            filled.
        modified (models.DateTimeField): Required creation datetime, automatically
            filled.
        title (models.CharField): Required unique title string.
        slug (models.CharField): Required unique slug string.
    """
    consumable = models.ForeignKey(
        "atoum.consumable",
        verbose_name=_("Consumable"),
        on_delete=models.CASCADE
    )
    created = models.DateTimeField(
        _("creation date"),
        db_index=True,
        default=timezone.now,
    )
    modified = models.DateTimeField(
        _("modification date"),
        db_index=True,
        default=timezone.now,
    )
    title = models.CharField(
        _("title"),
        blank=False,
        max_length=100,
        default="",
        unique=True,
    )
    slug = models.CharField(
        _("slug"),
        blank=False,
        max_length=130,
        default="",
        unique=True,
    )

    COMMON_ORDER_BY = ["title"]
    """
    List of field names for ordering queryset related to the same consumable.
    """

    HIERARCHY_SELECT_RELATED = ["consumable"]
    """
    List of foreign-key relationships field names to "follow" in queryset to avoid
    multiple queries. Commonly used in ``Queryset.select_related()``.
    """

    HIERARCHY_ORDER = [
        "consumable__title",
        "title"
    ]
    """
    List of field names for ordering queryset respecting relationships. This is to be
    used when listing objects related to mixed consumables.
    """

    class Meta:
        verbose_name = _("Assortment")
        verbose_name_plural = _("Assortments")
        ordering = [
            "title",
        ]

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        """
        Return absolute URL to the detail view.

        Returns:
            string: An URL.
        """
        return reverse("atoum:assortment-detail", kwargs={
            "consumable_slug": self.consumable.slug,
            "assortment_slug": self.slug,
        })

    def parenting_crumbs(self):
        """
        Return parenting crumbs from Consumable to Assortment.

        Returns:
            list: List of crumb titles in order.
        """
        return [
            self.consumable.title,
            self.title
        ]

    def parenting_crumbs_html(self):
        """
        Display HTML of Assortment label with its parent consumable.

        Returns:
            string: A label such as ``Consumable > Assortment``.
        """
        return format_html("{0} &gt; {1}", *self.parenting_crumbs())

    def get_categories(self):
        """
        Return a queryset for categories related to the assortment.

        Returns:
            Queryset:
        """
        return self.category_set.all().prefetch_related("product_set")

    def save(self, *args, **kwargs):
        # Auto update 'modified' value on each save
        self.modified = timezone.now()

        super().save(*args, **kwargs)



import factory

from ..models import Assortment
from .consumable import ConsumableFactory


class AssortmentFactory(factory.django.DjangoModelFactory):
    """
    Factory to create instance of a Assortment.
    """
    consumable = factory.SubFactory(ConsumableFactory)
    title = factory.Sequence(lambda n: "Assortment {0}".format(n))
    slug = factory.Sequence(lambda n: "assortment-{0}".format(n))

    class Meta:
        model = Assortment



from django import forms

from dal import autocomplete

from ..models import Product, ShoppingItem


class ProductBreadcrumbChoiceField(forms.ModelChoiceField):
    """
    Customize product model choice field.
    """
    def label_from_instance(self, obj):
        """
        Display product parenting crumbs of a choice option.

        This label should be the same returned from
        ``ProductAutocompleteView.get_selected_result_label()``.
        """
        return obj.parenting_crumbs_html()


class ShoppingItemInlineForm(autocomplete.FutureModelForm):
    class Meta:
        fields = "__all__"
        model = ShoppingItem

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Override Product model form field to customize option label and enable
        # DAL autocompletion
        self.fields["product"] = ProductBreadcrumbChoiceField(
            queryset=Product.objects.all().select_related(
                "category",
                "category__assortment",
                "category__assortment__consumable"
            ),
            required=True,
            blank=False,
            widget=autocomplete.ModelSelect2(
                url="atoum:autocomplete-products",
                attrs={"data-html": True},
            ),
        )



from django.conf import settings
from django.contrib.auth.mixins import UserPassesTestMixin
from django.http import Http404, HttpResponseBadRequest
from django.views.generic import ListView
from django.views.generic.detail import SingleObjectMixin
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _

from dal import autocomplete

from ..models import Assortment
from .consumable import ConsumableDetailView
from .mixins import AtoumBreadcrumMixin


class AssortmentIndexView(AtoumBreadcrumMixin, ListView):
    """
    List of assortments
    """
    model = Assortment
    template_name = "atoum/assortment/index.html"
    paginate_by = settings.ASSORTMENT_PAGINATION
    crumb_title = _("Assortments")
    crumb_urlname = "atoum:assortment-index"

    def get_queryset(self):
        return self.model.objects.order_by("title").select_related(
            *Assortment.HIERARCHY_SELECT_RELATED
        )

    @property
    def crumbs(self):
        return [
            (
                AssortmentIndexView.crumb_title,
                reverse(AssortmentIndexView.crumb_urlname)
            ),
        ]


class AssortmentDetailView(AtoumBreadcrumMixin, SingleObjectMixin, ListView):
    """
    Assortment detail and its related category list
    """
    template_name = "atoum/assortment/detail.html"
    paginate_by = settings.CATEGORY_PAGINATION
    context_object_name = "assortment_object"
    crumb_title = None  # No usage since title depends from object
    crumb_urlname = "atoum:assortment-detail"

    @property
    def crumbs(self):
        return [
            (
                self.object.consumable.title,
                reverse(ConsumableDetailView.crumb_urlname, kwargs={
                    "slug": self.object.consumable.slug,
                })
            ),
            (
                self.object.title,
                reverse(self.crumb_urlname, kwargs={
                    "consumable_slug": self.object.consumable.slug,
                    "assortment_slug": self.object.slug,
                })
            ),
        ]

    def get_queryset(self):
        """
        Queryset to list sub relations
        """
        return self.object.category_set.order_by("title")

    def get_object(self):
        """
        Get the Assortment object for details
        """
        consumable_slug = self.kwargs.get("consumable_slug")
        assortment_slug = self.kwargs.get("assortment_slug")

        try:
            obj = Assortment.objects.filter(**{
                "consumable__slug": consumable_slug,
                "slug": assortment_slug,
            }).select_related(*Assortment.HIERARCHY_SELECT_RELATED).get()
        except Assortment.DoesNotExist:
            raise Http404(
                _("No {} found matching the query").format(
                    Assortment._meta.verbose_name
                )
            )

        return obj

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()

        return super().get(request, *args, **kwargs)


class AssortmentAutocompleteView(UserPassesTestMixin, autocomplete.Select2QuerySetView):
    """
    View for Assortment autocompletion list with DAL.
    """
    def test_func(self):
        """
        Limit to admin only
        """
        return self.request.user.is_staff

    def get_queryset(self):
        """
        Build list queryset.

        Enable ``select_related`` on relation ``consumable`` because we intend to use
        it commonly in labels.
        """
        if not self.request.user.is_authenticated:
            return Assortment.objects.none()

        qs = Assortment.objects.all()

        if self.q:
            qs = qs.filter(title__istartswith=self.q).select_related(
                *Assortment.HIERARCHY_SELECT_RELATED
            )

        return qs.order_by(*Assortment.HIERARCHY_ORDER)

    def get_result_label(self, result):
        """
        Format a better list result display.
        """
        return format_html(
            "<small>{consumable}</small><br>{assortment}",
            consumable=result.consumable.title,
            assortment=result.title,
        )

    def get_selected_result_label(self, result):
        """
        Format a better selected result display.
        """
        return result.parenting_crumbs_html()

    def post(self, request, *args, **kwargs):
        """
        POST request is forbidden since DAL would allow creation of new object for a
        missing value and we don't want it.
        """
        return HttpResponseBadRequest()



from django.db import models
from django.db.models.signals import post_delete, pre_save
from django.utils.translation import gettext_lazy as _
from django.urls import reverse
from django.utils import timezone

from smart_media.mixins import SmartFormatMixin
from smart_media.modelfields import SmartMediaField
from smart_media.signals import auto_purge_files_on_change, auto_purge_files_on_delete


class Brand(SmartFormatMixin, models.Model):
    """
    Brand of products.

    Attributes:
        created (models.DateTimeField): Required creation datetime, automatically
            filled.
        modified (models.DateTimeField): Required creation datetime, automatically
            filled.
        title (models.CharField): Required unique title string.
        slug (models.CharField): Required unique slug string.
        description (models.TextField): Optional description long string.
        cover (SmartMediaField): Optional cover image file.
    """
    created = models.DateTimeField(
        _("creation date"),
        db_index=True,
        default=timezone.now,
    )
    modified = models.DateTimeField(
        _("modification date"),
        db_index=True,
        default=timezone.now,
    )
    title = models.CharField(
        _("title"),
        blank=False,
        max_length=100,
        default="",
        unique=True,
    )
    slug = models.CharField(
        _("slug"),
        blank=False,
        max_length=130,
        default="",
        unique=True,
    )
    description = models.TextField(
        _("description"),
        blank=True,
        default="",
    )
    cover = SmartMediaField(
        verbose_name=_("cover image"),
        upload_to="atoum/brand/cover/%y/%m",
        max_length=255,
        blank=True,
        default="",
    )

    COMMON_ORDER_BY = ["title"]
    """
    List of field order commonly used in frontend view/api
    """

    class Meta:
        verbose_name = _("Brand")
        verbose_name_plural = _("Brands")
        ordering = [
            "title",
        ]

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        """
        Return absolute URL to the detail view.

        Returns:
            string: An URL.
        """
        return reverse("atoum:brand-detail", args=[self.slug])

    def save(self, *args, **kwargs):
        # Auto update 'modified' value on each save
        self.modified = timezone.now()

        super().save(*args, **kwargs)


# Connect some signals
post_delete.connect(
    auto_purge_files_on_delete(["cover"]),
    dispatch_uid="brand_cover_on_delete",
    sender=Brand,
    weak=False,
)
pre_save.connect(
    auto_purge_files_on_change(["cover"]),
    dispatch_uid="brand_cover_on_change",
    sender=Brand,
    weak=False,
)



# Generated by Django 5.0.9 on 2025-01-02 14:23

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("atoum", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Shopping",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        verbose_name="creation date",
                    ),
                ),
                (
                    "planning",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        verbose_name="planning date",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        default="", max_length=100, unique=True, verbose_name="title"
                    ),
                ),
            ],
            options={
                "verbose_name": "Shopping",
                "verbose_name_plural": "Shoppings",
                "ordering": ["title"],
            },
        ),
        migrations.CreateModel(
            name="ShoppingItem",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name="creation date"
                    ),
                ),
                ("quantity", models.PositiveSmallIntegerField(verbose_name="quantity")),
                (
                    "done",
                    models.BooleanField(blank=True, default=False, verbose_name="done"),
                ),
                (
                    "product",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="atoum.product"
                    ),
                ),
                (
                    "shopping",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="atoum.shopping"
                    ),
                ),
            ],
            options={
                "verbose_name": "Shopping item",
                "verbose_name_plural": "Shopping items",
                "ordering": ["shopping", "product"],
            },
        ),
        migrations.AddField(
            model_name="shopping",
            name="products",
            field=models.ManyToManyField(
                blank=True,
                related_name="shoppings",
                through="atoum.ShoppingItem",
                to="atoum.product",
                verbose_name="items",
            ),
        ),
        migrations.DeleteModel(
            name="Article",
        ),
        migrations.DeleteModel(
            name="Blog",
        ),
        migrations.AddConstraint(
            model_name="shoppingitem",
            constraint=models.UniqueConstraint(
                fields=("shopping", "product"),
                name="atoum_unique_shoppingitem_shopping_product",
            ),
        ),
    ]



# Generated by Django 5.0.9 on 2025-01-02 16:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("atoum", "0002_shopping_shoppingitem_shopping_products_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="shopping",
            name="title",
            field=models.CharField(
                blank=True,
                default="",
                max_length=100,
                unique=True,
                verbose_name="title",
            ),
        ),
    ]



import factory

from django.apps import apps
from django.conf import settings
from django.db.models.signals import post_save


def safe_get_user_model():
    """
    Safe loading of the User model, customized or not.
    """
    user_app, user_model = settings.AUTH_USER_MODEL.split('.')
    return apps.get_registered_model(user_app, user_model)


@factory.django.mute_signals(post_save)
class UserFactory(factory.django.DjangoModelFactory):
    """
    Factory to create an User object.
    """
    first_name = factory.Faker("first_name")
    last_name = factory.Faker("last_name")
    username = factory.Sequence(lambda n: "demo-user-%d" % n)
    is_active = True
    is_staff = False
    is_superuser = False
    password = "secret"

    class Meta:
        model = safe_get_user_model()
        skip_postgeneration_save = True

    class Params:
        """
        Declare traits that add relevant parameters for admin and superuser
        """
        flag_is_admin = factory.Trait(
            is_superuser=False,
            is_staff=True,
            username=factory.Sequence(lambda n: "admin-%d" % n),
        )
        flag_is_superuser = factory.Trait(
            is_superuser=True,
            is_staff=True,
            username=factory.Sequence(lambda n: "superuser-%d" % n),
        )

    @factory.lazy_attribute
    def email(self):
        """
        Email is automatically build from username
        """
        return "%s@test.com" % self.username

    @classmethod
    def _create(cls, model_class, *args, **kwargs):
        """
        Ensure the raw password gets set after the initial save
        """
        password = kwargs.pop("password", None)
        obj = super(UserFactory, cls)._create(model_class, *args, **kwargs)
        obj.set_password(password)
        obj.save()
        return obj



from collections.abc import Iterable

from django.template import Library

register = Library()


@register.simple_tag(name="querystring", takes_context=True)
def querystring(context, query_dict=None, **kwargs):
    """
    Add, remove, and change parameters of a ``QueryDict`` and return the result
    as a query string. If the ``query_dict`` argument is not provided, default
    to ``request.GET``.

    .. Note::
       This is a portage of the Django 5.1 new tag so we can benefit from it with
       Django<5.0 until we drop their support.

    For example::

        {% querystring foo=3 %}

    To remove a key::

        {% querystring foo=None %}

    To use with pagination::

        {% querystring page=page_obj.next_page_number %}

    A custom ``QueryDict`` can also be used::

        {% querystring my_query_dict foo=3 %}
    """
    if query_dict is None:
        query_dict = context.request.GET
    query_dict = query_dict.copy()
    for key, value in kwargs.items():
        if value is None:
            if key in query_dict:
                del query_dict[key]
        elif isinstance(value, Iterable) and not isinstance(value, str):
            query_dict.setlist(key, value)
        else:
            query_dict[key] = value
    if not query_dict:
        return ""
    query_string = query_dict.urlencode()
    return f"?{query_string}"



from .assortment import AssortmentFactory
from .brand import BrandFactory
from .category import CategoryFactory
from .consumable import ConsumableFactory
from .product import ProductFactory
from .shopping import ShoppingFactory, ShoppingItemFactory
from .user import UserFactory


__all__ = [
    "AssortmentFactory",
    "BrandFactory",
    "CategoryFactory",
    "ConsumableFactory",
    "ProductFactory",
    "ShoppingFactory",
    "ShoppingItemFactory",
    "UserFactory",
]



from haystack.forms import ModelSearchForm

from ..models import Assortment, Category, Product
from ..form_helpers import AdvancedSearchFormHelper


class GlobalSearchForm(ModelSearchForm):
    """
    Form to search on all enabled Atoum models.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # We don't want label since we use a group inline layout
        self.fields["q"].label = False
        self.fields["models"].label = False

        self.helper = AdvancedSearchFormHelper()

    def search(self):
        """
        We don't keep the base queryset from inherit ModelSearchForm since it starts
        with an 'auto_query' that is not efficient with partial search. However this
        drops the feature of some operator like ``-`` to negate keyword.
        """
        if not self.is_valid():
            return self.no_query_found()

        if not self.cleaned_data.get("q"):
            return self.no_query_found()

        sqs = self.searchqueryset.filter(title_partial=self.cleaned_data["q"])

        # Enable all discovered model indexes from enabled applications
        sqs = sqs.models(*self.get_models())

        if self.load_all:
            # Get model objects from search result references
            sqs = sqs.load_all()

            # Define Assortment relationships to select in queryset instead of getting
            # them on their own querysets
            sqs = sqs.load_all_queryset(
                Assortment,
                Assortment.objects.all().select_related(
                    *Assortment.HIERARCHY_SELECT_RELATED
                )
            )

            # Define Category relationships to select
            sqs = sqs.load_all_queryset(
                Category,
                Category.objects.all().select_related(
                    *Category.HIERARCHY_SELECT_RELATED
                )
            )

            # Define Product relationships to select
            sqs = sqs.load_all_queryset(
                Product,
                Product.objects.all().select_related(
                    *Product.HIERARCHY_SELECT_RELATED
                )
            )

        return sqs



"""
Specific application exceptions.
"""


class AtoumBaseException(Exception):
    """
    Exception base.

    You should never use it directly except for test purpose. Instead make or
    use a dedicated exception related to the error context.
    """
    pass


class AppOperationError(AtoumBaseException):
    """
    Sample exception to raise from your code.
    """
    pass



from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from crispy_forms.layout import Field, Layout
from crispy_forms.bootstrap import FieldWithButtons, StrictButton

from .default import DefaultFormHelper
from .layout_objects import ListGroupCheckboxes


class AdvancedSearchFormHelper(DefaultFormHelper):
    """
    Advanced search form layout helper contains every available search fields.
    """
    DEFAULT_CSSID = "advanced-search"
    DEFAULT_METHOD = "get"
    DEFAULT_CSSCLASSES = "search-advanced-form needs-validation"

    def provide_action(self, value=None):
        self.form_action = reverse("atoum:search-results")

    def provide_layout(self, value=None, layout_args=None, layout_kwargs=None):
        self.layout = Layout(
            FieldWithButtons(
                Field("q", placeholder=_("Search"), required=""),
                StrictButton("Go!", type="submit", css_class="btn-primary"),
                input_size="input-group-sm"
            ),
            ListGroupCheckboxes(
                "models",
                group_class="list-group-horizontal-md",
                item_class="list-group-item-light",
            ),
        )



from .assortment import Assortment
from .brand import Brand
from .category import Category
from .consumable import Consumable
from .product import Product
from .shopping import Shopping, ShoppingItem


__all__ = [
    "Assortment",
    "Brand",
    "Category",
    "Consumable",
    "Product",
    "Shopping",
    "ShoppingItem",
]



# Generated by Django 5.0.9 on 2024-09-23 14:59

import django.db.models.deletion
import django.utils.timezone
import smart_media.modelfields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Assortment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        verbose_name="creation date",
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        verbose_name="modification date",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        default="", max_length=100, unique=True, verbose_name="title"
                    ),
                ),
                (
                    "slug",
                    models.CharField(
                        default="", max_length=130, unique=True, verbose_name="slug"
                    ),
                ),
            ],
            options={
                "verbose_name": "Assortment",
                "verbose_name_plural": "Assortments",
                "ordering": ["title"],
            },
        ),
        migrations.CreateModel(
            name="Blog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        default="", max_length=55, unique=True, verbose_name="title"
                    ),
                ),
            ],
            options={
                "verbose_name": "Blog",
                "verbose_name_plural": "Blogs",
                "ordering": ["title"],
            },
        ),
        migrations.CreateModel(
            name="Brand",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        verbose_name="creation date",
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        verbose_name="modification date",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        default="", max_length=100, unique=True, verbose_name="title"
                    ),
                ),
                (
                    "slug",
                    models.CharField(
                        default="", max_length=130, unique=True, verbose_name="slug"
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True, default="", verbose_name="description"
                    ),
                ),
                (
                    "cover",
                    smart_media.modelfields.SmartMediaField(
                        blank=True,
                        default="",
                        max_length=255,
                        upload_to="atoum/brand/cover/%y/%m",
                        verbose_name="cover image",
                    ),
                ),
            ],
            options={
                "verbose_name": "Brand",
                "verbose_name_plural": "Brands",
                "ordering": ["title"],
            },
        ),
        migrations.CreateModel(
            name="Consumable",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        verbose_name="creation date",
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        verbose_name="modification date",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        default="", max_length=100, unique=True, verbose_name="title"
                    ),
                ),
                (
                    "slug",
                    models.CharField(
                        default="", max_length=130, unique=True, verbose_name="slug"
                    ),
                ),
            ],
            options={
                "verbose_name": "Consumable",
                "verbose_name_plural": "Consumables",
                "ordering": ["title"],
            },
        ),
        migrations.CreateModel(
            name="Article",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(default="", max_length=150, verbose_name="title"),
                ),
                (
                    "content",
                    models.TextField(blank=True, default="", verbose_name="content"),
                ),
                (
                    "publish_start",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        verbose_name="publication start",
                    ),
                ),
                (
                    "blog",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="atoum.blog",
                        verbose_name="Related blog",
                    ),
                ),
            ],
            options={
                "verbose_name": "Article",
                "verbose_name_plural": "Articles",
                "ordering": ["-publish_start"],
            },
        ),
        migrations.CreateModel(
            name="Category",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        verbose_name="creation date",
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        verbose_name="modification date",
                    ),
                ),
                (
                    "title",
                    models.CharField(default="", max_length=100, verbose_name="title"),
                ),
                (
                    "slug",
                    models.CharField(default="", max_length=130, verbose_name="slug"),
                ),
                (
                    "assortment",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="atoum.assortment",
                        verbose_name="Assortment",
                    ),
                ),
            ],
            options={
                "verbose_name": "Category",
                "verbose_name_plural": "Categories",
                "ordering": ["title"],
            },
        ),
        migrations.AddField(
            model_name="assortment",
            name="consumable",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="atoum.consumable",
                verbose_name="Consumable",
            ),
        ),
        migrations.CreateModel(
            name="Product",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        verbose_name="creation date",
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        verbose_name="modification date",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        default="", max_length=100, unique=True, verbose_name="title"
                    ),
                ),
                (
                    "slug",
                    models.CharField(
                        default="", max_length=130, unique=True, verbose_name="slug"
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True, default="", verbose_name="description"
                    ),
                ),
                (
                    "cover",
                    smart_media.modelfields.SmartMediaField(
                        blank=True,
                        default="",
                        max_length=255,
                        upload_to="atoum/product/cover/%y/%m",
                        verbose_name="cover image",
                    ),
                ),
                (
                    "brand",
                    models.ForeignKey(
                        blank=True,
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="atoum.brand",
                        verbose_name="brand",
                    ),
                ),
                (
                    "category",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="atoum.category",
                        verbose_name="Category",
                    ),
                ),
            ],
            options={
                "verbose_name": "Product",
                "verbose_name_plural": "Products",
                "ordering": ["title"],
            },
        ),
        migrations.AddConstraint(
            model_name="category",
            constraint=models.UniqueConstraint(
                fields=("assortment", "title"), name="atoum_unique_cat_assortment_title"
            ),
        ),
        migrations.AddConstraint(
            model_name="category",
            constraint=models.UniqueConstraint(
                fields=("assortment", "slug"), name="atoum_unique_cat_assortment_slug"
            ),
        ),
    ]



# Generated by Django 5.0.9 on 2025-01-06 16:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("atoum", "0003_alter_shopping_title"),
    ]

    operations = [
        migrations.AddField(
            model_name="shopping",
            name="done",
            field=models.BooleanField(blank=True, default=False, verbose_name="done"),
        ),
    ]



from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from import_export import resources
from import_export.admin import ImportExportModelAdmin

from ..models import Consumable
from .assortment import AssortmentInline


class ConsumableResource(resources.ModelResource):
    """
    Data resource configuration for 'django-import-export'.
    """
    class Meta:
        model = "atoum.Consumable"
        fields = ("title", "slug")
        import_id_fields = ("slug",)


@admin.register(Consumable)
class ConsumableAdmin(ImportExportModelAdmin):
    """
    Model admin controller.
    """
    readonly_fields = ["created", "modified"]
    prepopulated_fields = {
        "slug": ("title",),
    }
    ordering = Consumable.COMMON_ORDER_BY
    search_fields = [
        "title",
    ]
    list_display = (
        "title",
        "count_assortments",
        "modified",
    )
    inlines = [
        AssortmentInline,
    ]
    resource_classes = [ConsumableResource]

    def count_assortments(self, obj):
        """
        Count related assortments.
        """
        return obj.assortment_set.count()
    count_assortments.short_description = _("Assortments")



from django.db import models
from django.utils.translation import gettext_lazy as _
from django.urls import reverse
from django.utils import timezone


class Shopping(models.Model):
    """
    Shopping object to gather choices of products.

    Attributes:
        created (models.DateTimeField): Required creation datetime, automatically
            filled.
        planning (models.DateTimeField): Required planning datetime, automatically
            filled.
        title (models.CharField): Optional title string.
        done (models.CharField): Optional boolean.
        products (models.ManyToManyField): Optional product selection
    """
    created = models.DateTimeField(
        _("creation date"),
        db_index=True,
        default=timezone.now,
    )
    planning = models.DateTimeField(
        _("planning date"),
        db_index=True,
        default=timezone.now,
    )
    title = models.CharField(
        _("title"),
        blank=True,
        max_length=100,
        default="",
    )
    done = models.BooleanField(
        verbose_name=_("done"),
        default=False,
        blank=True,
    )
    products = models.ManyToManyField(
        "atoum.Product",
        verbose_name=_("items"),
        related_name="shoppings",
        blank=True,
        through="atoum.ShoppingItem",
        through_fields=("shopping", "product"),
    )

    COMMON_ORDER_BY = ["-planning"]
    """
    List of field order commonly used in frontend view/api
    """

    class Meta:
        verbose_name = _("Shopping")
        verbose_name_plural = _("Shoppings")
        ordering = [
            "title",
        ]

    def __str__(self):
        """
        TODO: We should return a localized and humanized date when falling back to
        planning date because of empty title.
        """
        return self.title or str(self.planning)

    def get_absolute_url(self):
        """
        Return absolute URL to the detail view.

        Returns:
            string: An URL.
        """
        return reverse("atoum:shopping-detail", args=[self.id])

    def save(self, *args, **kwargs):
        # Auto update 'modified' value on each save
        self.modified = timezone.now()

        super().save(*args, **kwargs)


class ShoppingItem(models.Model):
    """
    Shopping list item object.

    .. todo::
        Quantity value may be a choice field if product unit was a volume or weight.

    Attributes:
        created (models.DateTimeField): Required creation datetime, automatically
            filled.
        shopping (atoum.models.Shopping): Required
        product (atoum.models.Product): Required
        quantity (models.PositiveSmallIntegerField): Required positive small integer.
        done (models.CharField): Optional boolean.
    """
    created = models.DateTimeField(
        _("creation date"),
        default=timezone.now,
    )
    shopping = models.ForeignKey(
        "atoum.Shopping",
        on_delete=models.CASCADE
    )
    product = models.ForeignKey(
        "atoum.Product",
        on_delete=models.CASCADE
    )
    quantity = models.PositiveSmallIntegerField(_("quantity"))
    done = models.BooleanField(
        verbose_name=_("done"),
        default=False,
        blank=True,
    )

    class Meta:
        verbose_name = _("Shopping item")
        verbose_name_plural = _("Shopping items")
        ordering = [
            "shopping", "product",
        ]
        constraints = [
            # Enforce unique couple shopping + product
            models.UniqueConstraint(
                fields=[
                    "shopping", "product"
                ],
                name="atoum_unique_shoppingitem_shopping_product"
            ),
        ]



"""
Since ``Django 3.8``, ZoneInfo is the preferred way to localize date and play with
timezone instead of Pytz.

This module should safely load ``zoneinfo`` library to import ``ZoneInfo`` class
for ``Django>=3.8``.
"""
try:
    from zoneinfo import ZoneInfo  # noqa: F401,F403
# Django 4.x install the backports for Python 3.8
except ModuleNotFoundError:
    from backports.zoneinfo import ZoneInfo  # noqa: F401,F403



from django.db import models
from django.utils.translation import gettext_lazy as _
from django.urls import reverse
from django.utils import timezone


class Consumable(models.Model):
    """
    The very top level of consumable classification.

    Attributes:
        created (models.DateTimeField): Required creation datetime, automatically
            filled.
        modified (models.DateTimeField): Required creation datetime, automatically
            filled.
        title (models.CharField): Required unique title string.
        slug (models.CharField): Required unique slug string.
    """
    created = models.DateTimeField(
        _("creation date"),
        db_index=True,
        default=timezone.now,
    )
    modified = models.DateTimeField(
        _("modification date"),
        db_index=True,
        default=timezone.now,
    )
    title = models.CharField(
        _("title"),
        blank=False,
        max_length=100,
        default="",
        unique=True,
    )
    slug = models.CharField(
        _("slug"),
        blank=False,
        max_length=130,
        default="",
        unique=True,
    )

    COMMON_ORDER_BY = ["title"]
    """
    List of field order commonly used in frontend view/api
    """

    class Meta:
        verbose_name = _("Consumable")
        verbose_name_plural = _("Consumables")
        ordering = [
            "title",
        ]

    def __str__(self):
        return self.title

    def get_assortments(self):
        """
        Return a queryset for assortment related to the consumable.

        Returns:
            Queryset:
        """
        return self.assortment_set.all()

    def get_absolute_url(self):
        """
        Return absolute URL to the detail view.

        Returns:
            string: An URL.
        """
        return reverse("atoum:consumable-detail", kwargs={
            "slug": self.slug,
        })

    def save(self, *args, **kwargs):
        # Auto update 'modified' value on each save
        self.modified = timezone.now()

        super().save(*args, **kwargs)



# Generated by Django 5.0.9 on 2025-01-18 01:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("atoum", "0005_alter_shopping_title"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="shopping",
            name="products",
        ),
    ]



from .assortment import AssortmentAdmin, AssortmentResource
from .brand import BrandAdmin
from .category import CategoryAdmin, CategoryResource
from .consumable import ConsumableAdmin, ConsumableResource
from .product import ProductAdmin, ProductAdminInline, ProductResource
from .shopping import ShoppingAdmin


__all__ = [
    "AssortmentAdmin",
    "AssortmentResource",
    "BrandAdmin",
    "CategoryAdmin",
    "CategoryResource",
    "ConsumableResource",
    "ConsumableAdmin",
    "ProductAdmin",
    "ProductResource",
    "ProductAdminInline",
    "ShoppingAdmin",
]



from django.contrib import admin

from import_export import resources
from import_export.admin import ImportExportModelAdmin
from import_export import fields
from import_export.widgets import ForeignKeyWidget
from smart_media.admin import SmartAdminMixin

from ..forms import ProductAdminForm
from ..models import Category, Product


class ProductResource(resources.ModelResource):
    """
    Data resource configuration for 'django-import-export'.
    """
    category = fields.Field(
        column_name="category",
        attribute="category",
        widget=ForeignKeyWidget(Category, field="title")
    )

    class Meta:
        model = "atoum.Product"
        fields = ("title", "slug", "category", "description")
        import_id_fields = ("slug",)


@admin.register(Product)
class ProductAdmin(SmartAdminMixin, ImportExportModelAdmin):
    """
    Model admin controller.
    """
    form = ProductAdminForm
    list_select_related = ["category"]
    readonly_fields = ["created", "modified"]
    prepopulated_fields = {
        "slug": ("title",),
    }
    ordering = Product.COMMON_ORDER_BY
    search_fields = [
        "title",
        "description",
    ]
    list_display = (
        "title",
        "category",
        "brand",
        "modified",
    )
    autocomplete_fields = ["brand"]
    list_filter = (
        "category",
        "brand",
    )
    resource_classes = [ProductResource]


class ProductAdminInline(admin.StackedInline):
    """
    Inline model admin controller to be used in other model admin controllers.
    """
    model = Product
    autocomplete_fields = ["category", "brand"]
    exclude = ["created", "modified", "description"]
    prepopulated_fields = {
        "slug": ("title",),
    }
    extra = 0
    can_delete = False



from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from import_export import resources
from import_export.admin import ImportExportModelAdmin
from import_export import fields
from import_export.widgets import ForeignKeyWidget

from ..forms import CategoryAdminForm
from ..models import Assortment, Category
from .product import ProductAdminInline


class CategoryResource(resources.ModelResource):
    """
    Data resource configuration for 'django-import-export'.
    """
    assortment = fields.Field(
        column_name="assortment",
        attribute="assortment",
        widget=ForeignKeyWidget(Assortment, field="title")
    )

    class Meta:
        model = "atoum.Category"
        fields = ("title", "slug", "assortment")
        import_id_fields = ("slug", "assortment")


@admin.register(Category)
class CategoryAdmin(ImportExportModelAdmin):
    """
    Model admin controller.
    """
    form = CategoryAdminForm
    list_select_related = ["assortment"]
    readonly_fields = ["created", "modified"]
    prepopulated_fields = {
        "slug": ("title",),
    }
    ordering = Category.COMMON_ORDER_BY
    search_fields = [
        "title",
    ]
    list_display = (
        "title",
        "assortment",
        "count_products",
        "modified",
    )
    list_filter = (
        "assortment",
    )
    inlines = [
        ProductAdminInline,
    ]
    resource_classes = [CategoryResource]

    def count_products(self, obj):
        """
        Count related products.
        """
        return obj.product_set.count()
    count_products.short_description = _("Products")


class CategoryInline(admin.StackedInline):
    """
    Inline model admin controller to be used in other model admin controllers.
    """
    model = Category
    exclude = ["created", "modified"]
    prepopulated_fields = {
        "slug": ("title",),
    }
    extra = 0
    can_delete = False



from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from import_export import resources
from import_export.admin import ImportExportModelAdmin
from import_export import fields
from import_export.widgets import ForeignKeyWidget

from ..models import Assortment, Consumable
from .category import CategoryInline


class AssortmentResource(resources.ModelResource):
    """
    Data resource configuration for 'django-import-export'.
    """
    consumable = fields.Field(
        column_name="consumable",
        attribute="consumable",
        widget=ForeignKeyWidget(Consumable, field="title")
    )

    class Meta:
        model = "atoum.Assortment"
        fields = ("title", "slug", "consumable")
        import_id_fields = ("slug",)


@admin.register(Assortment)
class AssortmentAdmin(ImportExportModelAdmin):
    """
    Model admin controller.
    """
    list_select_related = ["consumable"]
    readonly_fields = ["created", "modified"]
    prepopulated_fields = {
        "slug": ("title",),
    }
    ordering = Assortment.COMMON_ORDER_BY
    search_fields = [
        "title",
    ]
    list_display = (
        "title",
        "consumable",
        "count_categories",
        "modified",
    )
    list_filter = (
        "consumable",
    )
    autocomplete_fields = ["consumable"]
    inlines = [
        CategoryInline,
    ]
    resource_classes = [AssortmentResource]

    def count_categories(self, obj):
        """
        Count related categories.
        """
        return obj.category_set.count()
    count_categories.short_description = _("Categories")


class AssortmentInline(admin.StackedInline):
    """
    Inline model admin controller to be used in other model admin controllers.
    """
    model = Assortment
    exclude = ["created", "modified"]
    prepopulated_fields = {
        "slug": ("title",),
    }
    extra = 0
    can_delete = False



from django.db import models
from django.utils.translation import gettext_lazy as _
from django.urls import reverse
from django.utils import timezone
from django.utils.html import format_html


class Category(models.Model):
    """
    Category of a consumable assortment.

    Attributes:
        assortment (models.ForeignKey): Required Assortment object.
        created (models.DateTimeField): Required creation datetime, automatically
            filled.
        modified (models.DateTimeField): Required creation datetime, automatically
            filled.
        title (models.CharField): Required unique title string.
        slug (models.CharField): Required unique slug string.
    """
    assortment = models.ForeignKey(
        "atoum.assortment",
        verbose_name=_("Assortment"),
        on_delete=models.CASCADE
    )
    created = models.DateTimeField(
        _("creation date"),
        db_index=True,
        default=timezone.now,
    )
    modified = models.DateTimeField(
        _("modification date"),
        db_index=True,
        default=timezone.now,
    )
    title = models.CharField(
        _("title"),
        blank=False,
        max_length=100,
        default="",
    )
    slug = models.CharField(
        _("slug"),
        blank=False,
        max_length=130,
        default="",
    )

    COMMON_ORDER_BY = ["title"]
    """
    List of field names for ordering queryset related to the same assortment.
    """

    HIERARCHY_SELECT_RELATED = [
        "assortment",
        "assortment__consumable"
    ]
    """
    List of foreign-key relationships field names to "follow" in queryset to avoid
    multiple queries. Commonly used in ``Queryset.select_related()``.
    """

    HIERARCHY_ORDER = [
        "assortment__consumable__title",
        "assortment__title",
        "title"
    ]
    """
    List of field names for ordering queryset respecting relationships. This is to be
    used when listing objects related to mixed assortments.
    """

    class Meta:
        verbose_name = _("Category")
        verbose_name_plural = _("Categories")
        ordering = [
            "title",
        ]
        constraints = [
            # Enforce unique couple assortment + title
            models.UniqueConstraint(
                fields=[
                    "assortment", "title"
                ],
                name="atoum_unique_cat_assortment_title"
            ),
            # Enforce unique couple assortment + slug
            models.UniqueConstraint(
                fields=[
                    "assortment", "slug"
                ],
                name="atoum_unique_cat_assortment_slug"
            ),
        ]

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        """
        Return absolute URL to the detail view.

        Returns:
            string: An URL.
        """
        return reverse("atoum:category-detail", kwargs={
            "consumable_slug": self.assortment.consumable.slug,
            "assortment_slug": self.assortment.slug,
            "category_slug": self.slug,
        })

    def parenting_crumbs(self):
        """
        Return parenting crumbs from Consumable to Assortment to Category.

        Returns:
            list: List of crumb titles in order.
        """
        return [
            self.assortment.consumable.title,
            self.assortment.title,
            self.title
        ]

    def parenting_crumbs_html(self):
        """
        Display HTML of Category label with its parent consumable then assortment.

        Returns:
            string: A label such as ``Consumable > Assortment > Category``.
        """
        return format_html("{0} &gt; {1} &gt; {2}", *self.parenting_crumbs())

    def get_products(self):
        """
        Return a queryset for products related to the category.

        Returns:
            Queryset:
        """
        return self.product_set.all()

    def save(self, *args, **kwargs):
        # Auto update 'modified' value on each save
        self.modified = timezone.now()

        super().save(*args, **kwargs)



from haystack import indexes

from .models import Assortment, Category, Consumable, Product


class ProductIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.CharField(
        document=True,
        use_template=True,
        template_name="atoum/search/base_field_indexes_template.txt"
    )
    title_partial = indexes.EdgeNgramField(model_attr="title")

    def get_model(self):
        return Product


class AssortmentIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.CharField(
        document=True,
        use_template=True,
        template_name="atoum/search/base_field_indexes_template.txt"
    )
    title_partial = indexes.EdgeNgramField(model_attr="title")

    def get_model(self):
        return Assortment


class ConsumableIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.CharField(
        document=True,
        use_template=True,
        template_name="atoum/search/base_field_indexes_template.txt"
    )
    title_partial = indexes.EdgeNgramField(model_attr="title")

    def get_model(self):
        return Consumable


class CategoryIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.CharField(
        document=True,
        use_template=True,
        template_name="atoum/search/base_field_indexes_template.txt"
    )
    title_partial = indexes.EdgeNgramField(model_attr="title")

    def get_model(self):
        return Category



from .assortment import (
    AssortmentAutocompleteView, AssortmentDetailView, AssortmentIndexView,
)
from .category import (
    CategoryAutocompleteView, CategoryDetailView, CategoryIndexView,
)
from .consumable import ConsumableDetailView, ConsumableIndexView
from .dashboard import DashboardView
from .dummy import DummyView
from .product import (
    ProductAutocompleteView, ProductDetailView, ProductIndexView,
)
from .search import GlobalSearchView
from .tree import RecursiveTreeView


__all__ = [
    "AssortmentAutocompleteView",
    "AssortmentDetailView",
    "AssortmentIndexView",
    "CategoryAutocompleteView",
    "CategoryDetailView",
    "CategoryIndexView",
    "ConsumableDetailView",
    "ConsumableIndexView",
    "DashboardView",
    "DummyView",
    "GlobalSearchView",
    "ProductAutocompleteView",
    "ProductDetailView",
    "ProductIndexView",
    "RecursiveTreeView",
]



from django.views.generic import TemplateView


class DummyView(TemplateView):
    """
    Temporary dummy view that show nothing. Used for get_absolute_url from models until
    their detail view has been done.
    """
    template_name = "atoum/dummy.html"



# Generated by Django 5.0.9 on 2025-01-08 01:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("atoum", "0004_shopping_done"),
    ]

    operations = [
        migrations.AlterField(
            model_name="shopping",
            name="title",
            field=models.CharField(
                blank=True, default="", max_length=100, verbose_name="title"
            ),
        ),
    ]



from django.conf import settings
from django.views.generic import ListView
from django.views.generic.detail import SingleObjectMixin
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from ..models import Consumable
from .mixins import AtoumBreadcrumMixin


class ConsumableIndexView(AtoumBreadcrumMixin, ListView):
    """
    List of consumables
    """
    model = Consumable
    template_name = "atoum/consumable/index.html"
    paginate_by = None
    crumb_title = _("Consumables")
    crumb_urlname = "atoum:consumable-index"

    @property
    def crumbs(self):
        return [
            (
                ConsumableIndexView.crumb_title,
                reverse(ConsumableIndexView.crumb_urlname)
            ),
        ]

    def get_queryset(self):
        return self.model.objects.order_by("title")


class ConsumableDetailView(AtoumBreadcrumMixin, SingleObjectMixin, ListView):
    """
    Consumable detail and its related assortment list
    """
    template_name = "atoum/consumable/detail.html"
    paginate_by = settings.ASSORTMENT_PAGINATION
    context_object_name = "consumable_object"
    crumb_title = None  # No usage since title depends from object
    crumb_urlname = "atoum:consumable-detail"

    @property
    def crumbs(self):
        return [
            (
                self.object.title,
                reverse(self.crumb_urlname, kwargs={"slug": self.object.slug})
            ),
        ]

    def get_queryset(self):
        return self.object.assortment_set.order_by("title")

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=Consumable.objects.all())

        return super().get(request, *args, **kwargs)



"""
The old way to get a storage object has been deprecated since ``Django 4.2`` and
removed since ``Django 5.1``.

This modules should safely get and initialize the default storage class for
``Django>=3.8`` adn exposes it as a variable ``DEFAULT_STORAGE``.
"""
try:
    # Attempt to check for Django>=5.0 behavior
    from django.core.files.storage import storages  # noqa: F401,F403
except ImportError:
    # Fallback to Django<=4.2 behavior
    from django.core.files.storage import get_storage_class
    DEFAULT_STORAGE = get_storage_class()()  # noqa: F401,F403
else:
    # Result for Django>=5.0
    from django.conf import settings
    from django.utils.module_loading import import_string
    DEFAULT_STORAGE = import_string(
        settings.STORAGES["default"]["BACKEND"]
    )()  # noqa: F401,F403



from django.views.generic import ListView


from ..models import Consumable


class RecursiveTreeView(ListView):
    """
    Full recursive tree of Atoum objects (excepted Brand).

    .. WARNING::

        This recursive tree is highly innefficient on database queries.
        Usage of ``prefetch_related`` is not enough to avoid cascade of queryset for
        products.

        If this view is to be keeped, it will need to be optimized with a tree resolving
        that would gather objects of respectively all consumables, assortments,
        categories and then products.

    .. TODO::
        At least if we can make this more efficient and needs to keep it, make it
        restricted to staff users. And possibly cache it ?
    """
    model = Consumable
    template_name = "atoum/recursivetree.html"
    paginate_by = None

    def get_queryset(self):
        q = self.model.objects.all().prefetch_related("assortment_set")
        return q.order_by("title")



from crispy_forms.helper import FormHelper


class DefaultFormHelper(FormHelper):
    """
    Default form layout helper with Crispy forms.

    Attributes starting with ``DEFAULT_`` are the default values for helper options.
    Use it only for hardcoded values and prefer to fill them in respective ``provide_``
    method for evaluated values (like Layout object, url resolving, queryset, etc..).
    """
    DEFAULT_ACTION = "."
    DEFAULT_ATTRS = {"novalidate": ""}
    DEFAULT_CSSCLASSES = "needs-validation"
    DEFAULT_CSSID = None
    DEFAULT_ENABLETAG = True
    DEFAULT_INPUTS = None
    DEFAULT_LAYOUT = None
    DEFAULT_METHOD = "post"

    def __init__(self, *args, **kwargs):
        self.provide_attrs(kwargs.pop("attrs", None))
        self.provide_action(kwargs.pop("action", None))
        self.provide_cssclass(kwargs.pop("cssclass", None))
        self.provide_cssid(kwargs.pop("cssid", None))
        self.provide_tag(kwargs.pop("tag", None))
        self.provide_method(kwargs.pop("method", None))
        self.provide_layout(
            kwargs.pop("layout", None),
            layout_args=kwargs.pop("layout_args", None),
            layout_kwargs=kwargs.pop("layout_kwargs", None)
        )
        self.provide_inputs(kwargs.pop("inputs", None))

        super().__init__(*args, **kwargs)

    def provide_action(self, value=None):
        value = value or self.DEFAULT_ACTION
        if value:
            self.form_action = value

    def provide_attrs(self, value=None):
        value = value or self.DEFAULT_ATTRS
        if value:
            self.form_attrs = value

    def provide_cssclass(self, value=None):
        value = value or self.DEFAULT_CSSCLASSES
        if value:
            self.form_class = value

    def provide_cssid(self, value=None):
        value = value or self.DEFAULT_CSSID
        if value:
            self.form_id = value

    def provide_tag(self, value=None):
        value = value or self.DEFAULT_ENABLETAG
        if value:
            self.form_tag = value

    def provide_method(self, value=None):
        value = value or self.DEFAULT_METHOD
        if value:
            self.form_method = value

    def provide_inputs(self, value=None):
        value = value or self.DEFAULT_INPUTS
        if value:
            self.form_method = value
            for item in value:
                self.add_input(item)

    def provide_layout(self, value=None, layout_args=None, layout_kwargs=None):
        value = value or self.DEFAULT_LAYOUT
        if value:
            self.layout = value



import factory

from ..models import Product
from ..utils.imaging import DjangoSampleImageCrafter
from .brand import BrandFactory
from .category import CategoryFactory


class ProductFactory(factory.django.DjangoModelFactory):
    """
    Factory to create instance of a Product.
    """
    category = factory.SubFactory(CategoryFactory)
    brand = factory.SubFactory(BrandFactory)
    title = factory.Sequence(lambda n: "Product {0}".format(n))
    slug = factory.Sequence(lambda n: "product-{0}".format(n))
    description = factory.Faker("text", max_nb_chars=50)

    class Meta:
        model = Product

    @factory.lazy_attribute
    def cover(self):
        """
        Fill cover field with generated image.

        Returns:
            django.core.files.File: File object.
        """
        crafter = DjangoSampleImageCrafter()
        return crafter.create()



from django.db import models
from django.db.models.signals import post_delete, pre_save
from django.utils.translation import gettext_lazy as _
from django.urls import reverse
from django.utils import timezone
from django.utils.html import format_html

from smart_media.mixins import SmartFormatMixin
from smart_media.modelfields import SmartMediaField
from smart_media.signals import auto_purge_files_on_change, auto_purge_files_on_delete


class Product(SmartFormatMixin, models.Model):
    """
    Product of a Category.

    TODO: We should add a new field 'unit' that would be a choice of free(?), volume,
    weight. So shopping item can choose a proper quantity like 50L for a volume or
    2Kg for a weight and finally a free quantity amount for a free input.

    Attributes:
        category (models.ForeignKey): Required Assortment object.
        brand (models.ForeignKey): Optional Brand object.
        created (models.DateTimeField): Required creation datetime, automatically
            filled.
        modified (models.DateTimeField): Required creation datetime, automatically
            filled.
        title (models.CharField): Required unique title string.
        slug (models.CharField): Required unique slug string.
        description (models.TextField): Optional description long string.
        cover (SmartMediaField): Optional cover image file.
    """
    category = models.ForeignKey(
        "atoum.category",
        verbose_name=_("Category"),
        on_delete=models.CASCADE
    )
    brand = models.ForeignKey(
        "atoum.brand",
        verbose_name=_("brand"),
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        default=None,
    )
    created = models.DateTimeField(
        _("creation date"),
        db_index=True,
        default=timezone.now,
    )
    modified = models.DateTimeField(
        _("modification date"),
        db_index=True,
        default=timezone.now,
    )
    title = models.CharField(
        _("title"),
        blank=False,
        max_length=100,
        default="",
        unique=True,
    )
    slug = models.CharField(
        _("slug"),
        blank=False,
        max_length=130,
        default="",
        unique=True,
    )
    description = models.TextField(
        _("description"),
        blank=True,
        default="",
    )
    cover = SmartMediaField(
        verbose_name=_("cover image"),
        upload_to="atoum/product/cover/%y/%m",
        max_length=255,
        blank=True,
        default="",
    )

    COMMON_ORDER_BY = ["title"]
    """
    List of field names for ordering queryset related to the same assortment.
    """

    HIERARCHY_SELECT_RELATED = [
        "category",
        "category__assortment",
        "category__assortment__consumable",
    ]
    """
    List of foreign-key relationships field names to "follow" in queryset to avoid
    multiple queries. Commonly used in ``Queryset.select_related()``.
    """

    HIERARCHY_ORDER = [
        "category__assortment__consumable__title",
        "category__assortment__title",
        "category__title",
        "title"
    ]
    """
    List of field names for ordering queryset respecting relationships. This is to be
    used when listing assortments related to mixed consumables.
    """

    class Meta:
        verbose_name = _("Product")
        verbose_name_plural = _("Products")
        ordering = [
            "title",
        ]

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        """
        Return absolute URL to the detail view.

        Returns:
            string: An URL.
        """
        return reverse("atoum:product-detail", kwargs={
            "consumable_slug": self.category.assortment.consumable.slug,
            "assortment_slug": self.category.assortment.slug,
            "category_slug": self.category.slug,
            "product_slug": self.slug,
        })

    def parenting_crumbs(self):
        """
        Return parenting crumbs from Consumable to Assortment to Category to Product.

        Returns:
            list: List of crumb titles in order.
        """
        return [
            self.category.assortment.consumable.title,
            self.category.assortment.title,
            self.category.title,
            self.title
        ]

    def parenting_crumbs_html(self):
        """
        Display HTML of Product label with its parent consumable then assortment.

        Returns:
            string: A label such as ``Consumable > Assortment > Category > Product``.
        """
        return format_html("{0} &gt; {1} &gt; {2} &gt; {3}", *self.parenting_crumbs())

    def save(self, *args, **kwargs):
        # Auto update 'modified' value on each save
        self.modified = timezone.now()

        super().save(*args, **kwargs)


# Connect some signals
post_delete.connect(
    auto_purge_files_on_delete(["cover"]),
    dispatch_uid="product_cover_on_delete",
    sender=Product,
    weak=False,
)
pre_save.connect(
    auto_purge_files_on_change(["cover"]),
    dispatch_uid="product_cover_on_change",
    sender=Product,
    weak=False,
)



from django.contrib import admin

from ..forms import ShoppingItemInlineForm
from ..models import Shopping, ShoppingItem


class ShoppingItemInlineAdmin(admin.StackedInline):
    """
    Inline model admin controller for 'ShoppingItem' to be used in other model admin
    controllers.
    """
    form = ShoppingItemInlineForm
    model = ShoppingItem
    exclude = ["created", "modified"]
    extra = 0
    can_delete = True


@admin.register(Shopping)
class ShoppingAdmin(admin.ModelAdmin):
    """
    Model admin controller for 'Shopping'.
    """
    inlines = [
        ShoppingItemInlineAdmin,
    ]



from zoneinfo import ZoneInfo

import factory
from faker import Faker

from ..models import Shopping, ShoppingItem
from .product import ProductFactory


class ShoppingFactory(factory.django.DjangoModelFactory):
    """
    Factory to create instance of a Shopping.
    """
    title = factory.Sequence(lambda n: "Shopping {0}".format(n))
    done = False

    class Meta:
        model = Shopping
        skip_postgeneration_save = True

    @factory.lazy_attribute
    def planning(self):
        """
        Fill planning with a datetime this year after now.

        Returns:
            datetime.datetime: A datetime.
        """
        value = Faker().date_time_this_year(
            after_now=True,
            tzinfo=ZoneInfo("UTC"),
        )

        return value.replace()

    @factory.post_generation
    def fill_products(self, create, extracted, **kwargs):
        """
        Add products.

        Arguments:
            create (bool): True for create strategy, False for build strategy.
            extracted (object): If ``True``, will create a new random product
                object. If a list assume it's a list of tuple ``(Product, Dict)``
                for item to add. The ``Dict`` is a dictionnary for ``through_defaults``
                to use that should at least contains required ``quantity`` value.
        """
        # Do nothing for build strategy
        if not create or not extracted:
            return

        # Create a new random item adopting directly with the item factory
        if extracted is True:
            ShoppingItemFactory(shopping=self)
        # Take given item objects
        else:
            # Add items
            for product, through_defaults in extracted:
                self.products.add(product, through_defaults=through_defaults)


class ShoppingItemFactory(factory.django.DjangoModelFactory):
    """
    Factory to create instance of a ShoppingItem.
    """
    shopping = factory.SubFactory(ShoppingFactory)
    product = factory.SubFactory(ProductFactory)
    quantity = 1
    done = False

    class Meta:
        model = ShoppingItem



from crispy_forms.layout import Field
from crispy_forms.utils import TEMPLATE_PACK


class ListGroupCheckboxes(Field):
    """
    Layout object for rendering checkboxes in a *List group*.

    Opposed to other crispy layout fields this one do not include content into a wrapper
    'div' but in the 'ul' group. So its ``wrapper_class`` and ``css_class`` arguments
    are useless.

    Example:

        ListGroupCheckboxes("field_name", group_class="")

    Arguments:
        **kwargs: Additional attributes are converted into key="value", pairs. These
            attributes are added to the ``<ul>``.

    Keyword Arguments:
        group_class (string): CSS class to apply on ``<ul>`` group.
        item_class (string): CSS class to apply on all ``<li>`` items.
    """

    template = "%s/layout/listgroupcheckboxes.html"

    def __init__(self, *args, **kwargs):
        self.group_class = kwargs.pop("group_class", None)
        self.item_class = kwargs.pop("item_class", None)

        super().__init__(*args, **kwargs)

    def render(self, form, context, template_pack=TEMPLATE_PACK, **kwargs):
        return super().render(
            form,
            context,
            template_pack=template_pack,
            extra_context={
                "group_class": self.group_class,
                "item_class": self.item_class,
            },
        )



"""
Default application settings
----------------------------

These are the default settings you can override in your own project settings
right after the line which load the default app settings.

"""

ASSORTMENT_PAGINATION = 30
"""
Assortment per page limit for pagination, set it to ``None`` to disable
pagination.
"""

CATEGORY_PAGINATION = 25
"""
Category per page limit for pagination, set it to ``None`` to disable
pagination.
"""

PRODUCT_PAGINATION = 50
"""
Product per page limit for pagination, set it to ``None`` to disable
pagination.
"""

ATOUM_BREADCRUMBS_SHOW_HOME = True
"""
If breadcrumbs should include the site homepage entry or not. This is only working
in Atoum views.
"""



# Generated by Django 5.0.9 on 2025-01-18 01:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("atoum", "0006_remove_shopping_products"),
    ]

    operations = [
        migrations.AddField(
            model_name="shopping",
            name="products",
            field=models.ManyToManyField(
                blank=True,
                related_name="shoppings",
                through="atoum.ShoppingItem",
                to="atoum.product",
                verbose_name="items",
            ),
        ),
    ]



from django.urls import path

from .views import (
    AssortmentAutocompleteView,
    AssortmentDetailView,
    AssortmentIndexView,
    CategoryAutocompleteView,
    CategoryDetailView,
    CategoryIndexView,
    ConsumableDetailView,
    ConsumableIndexView,
    DashboardView,
    DummyView,
    GlobalSearchView,
    ProductAutocompleteView,
    ProductDetailView,
    ProductIndexView,
    RecursiveTreeView,
)


app_name = "atoum"


urlpatterns = [
    path("", DashboardView.as_view(), name="dashboard"),

    # Full recursive tree of everything
    path("tree/", RecursiveTreeView.as_view(), name="tree"),

    path(
        "consumables/",
        ConsumableIndexView.as_view(),
        name="consumable-index"
    ),
    path(
        "assortments/",
        AssortmentIndexView.as_view(),
        name="assortment-index"
    ),
    path(
        "categories/",
        CategoryIndexView.as_view(),
        name="category-index"
    ),
    path(
        "products/",
        ProductIndexView.as_view(),
        name="product-index"
    ),
    path(
        "consumables/<slug:slug>/",
        ConsumableDetailView.as_view(),
        name="consumable-detail"
    ),
    path(
        "consumables/<slug:consumable_slug>/<slug:assortment_slug>/",
        AssortmentDetailView.as_view(),
        name="assortment-detail"
    ),
    path(
        (
            "consumables/<slug:consumable_slug>/<slug:assortment_slug>/"
            "<slug:category_slug>/"
        ),
        CategoryDetailView.as_view(),
        name="category-detail"
    ),
    path(
        (
            "consumables/<slug:consumable_slug>/<slug:assortment_slug>/"
            "<slug:category_slug>/<slug:product_slug>/"
        ),
        ProductDetailView.as_view(),
        name="product-detail"
    ),

    # TODO: These are temporarily dummy views enabled to allow reversing until
    # implemented
    path(
        "brands/<slug:slug>/",
        DummyView.as_view(),
        name="brand-detail"
    ),

    # Autocomplete views for various models, only for staff users
    path(
        "autocomplete/assortments/",
        AssortmentAutocompleteView.as_view(),
        name="autocomplete-assortments",
    ),
    path(
        "autocomplete/categories/",
        CategoryAutocompleteView.as_view(),
        name="autocomplete-categories",
    ),
    path(
        "autocomplete/products/",
        ProductAutocompleteView.as_view(),
        name="autocomplete-products",
    ),

    path("search/", GlobalSearchView.as_view(), name="search-results"),
]



import json

from atoum.models import Assortment, Brand, Category, Consumable, Product

from dataclasses import dataclass, field


@dataclass
class InitialCatalog:
    """
    Class for keeping track of created catalog objects.

    .. Note::
        The tree methods purpose is mostly for debugging. Their behavior is to start
        from stored objects then recursively list their children directly from database
        instead of those from this dataclass because retrieving children is more
        difficult from store than from db.
    """
    assortments: dict[Assortment] = field(default_factory=dict)
    brands: dict[Brand] = field(default_factory=dict)
    consumables: dict[Consumable] = field(default_factory=dict)
    categories: dict[Category] = field(default_factory=dict)
    products: dict[Product] = field(default_factory=dict)

    def get_repr(self, value, attr=None):
        """
        Return representation of an object.

        Arguments:
            value (object): Model object.

        Keyword Arguments:
            attr (string): If given it will get representation from given attribute
                from given object. Else on default it uses the ``repr()`` function.
        """
        if attr:
            return getattr(value, attr)
        return repr(value)

    def get_product_tree(self, category, attr=None):
        """
        Get recursive tree of all Product objects.

        Keyword Arguments:
            category (Category): Category object to walk directly from database.
                If not given, default is to walk on all stored Category objects.
            attr (string): If given it will get representation from given attribute
                from given object. Else on default it uses the ``repr()`` function.
                The representation attribute is given to all recursive method so it
                must exists on all models.

        Returns:
            list: List of tuples, each tuple contains the object slug and null value,
            since product has no children but we want to conserve the same dump
            structure for all model.
        """
        products = (
            category.get_products()
            if category
            else self.products.values()
        )
        return [
            (self.get_repr(v, attr=attr), None)
            for v in products
        ]

    def get_category_tree(self, assortment=None, attr=None):
        """
        Get recursive tree of all Category objects.

        Keyword Arguments:
            assortment (Assortment): Assortment object to walk directly from database.
                If not given, default is to walk on all stored Assortment objects.
            attr (string): If given it will get representation from given attribute
                from given object. Else on default it uses the ``repr()`` function.
                The representation attribute is given to all recursive method so it
                must exists on all models.

        Returns:
            list: List of tuples, each tuple contains the object slug and its possible
                children products.
        """
        categories = (
            assortment.get_categories()
            if assortment
            else self.categories.values()
        )
        return [
            (self.get_repr(v, attr=attr), self.get_product_tree(v, attr=attr))
            for v in categories
        ]

    def get_assortment_tree(self, consumable=None, attr=None):
        """
        Get recursive tree of all Assortment objects.

        Keyword Arguments:
            consumable (Consumable): Consumable object to walk directly from database.
                If not given, default is to walk on all stored Consumable objects.
            attr (string): If given it will get representation from given attribute
                from given object. Else on default it uses the ``repr()`` function.
                The representation attribute is given to all recursive method so it
                must exists on all models.

        Returns:
            list: List of tuples, each tuple contains the object slug and its possible
                children categories.
        """
        assortments = (
            consumable.get_assortments()
            if consumable
            else self.assortments.values()
        )
        return [
            (self.get_repr(v, attr=attr), self.get_category_tree(v, attr=attr))
            for v in assortments
        ]

    def get_consumable_tree(self, attr=None):
        """
        Get recursive tree of all stored Consumable objects.

        Keyword Arguments:
            attr (string): If given it will get representation from given attribute
                from given object. Else on default it uses the ``repr()`` function.
                The representation attribute is given to all recursive method so it
                must exists on all models.

        Returns:
            list: List of tuples, each tuple contains the object slug and its possible
                children assortments.
        """
        return [
            (self.get_repr(v, attr=attr), self.get_assortment_tree(v, attr=attr))
            for v in self.consumables.values()
        ]

    def get_brand_tree(self, attr=None):
        """
        Get recursive tree of all stored Brand objects.

        .. Note::
            Opposed to other model objects, Brand is out the consumable tree and live
            along so it is not included in every "full tree" methods.

        .. Todo::
            Each brand should list its related products.

        Keyword Arguments:
            attr (string): If given it will get representation from given attribute
                from given object. Else on default it uses the ``repr()`` function.
                The representation attribute is given to all recursive method so it
                must exists on all models.

        Returns:
            list: List of tuples, each tuple contains the object slug and a null value
            to be consistant with other tree structure.
        """
        return [
            (self.get_repr(v, attr=attr), None)
            for v in self.brands.values()
        ]

    def get_json_tree(self, attr=None):
        """
        Return a JSON payload of the full tree starting from consumables.

        Keyword Arguments:
            attr (string): If given it will get representation from given attribute
                from given object. Else on default it uses the ``repr()`` function.
                The representation attribute is given to all recursive method so it
                must exists on all models.

        Returns:
            string: JSON payload.
        """
        return json.dumps(self.get_consumable_tree(attr=attr), indent=4)

    def _recursive_walk(self, nodes, items, level=1, attr=None):
        if items:
            indent = ("   " * level) + "└─"
            for key, children in items:
                nodes.append(indent + key)
                self._recursive_walk(nodes, children, level=level + 1)

    def get_ascii_tree(self, attr=None):
        """
        Build a basic recursive ascii tree starting from consumables.

        Keyword Arguments:
            attr (string): If given it will get representation from given attribute
                from given object. Else on default it uses the ``repr()`` function.
                The representation attribute is given to all recursive method so it
                must exists on all models.

        Returns:
            string: An ascii tree.
        """
        nodes = []
        for key, children in self.get_consumable_tree(attr=attr):
            nodes.append("───" + key)
            self._recursive_walk(nodes, children)

        return "\n".join(nodes)



from .category import CategoryAdminForm
from .product import ProductAdminForm
from .search import GlobalSearchForm
from .shopping import ShoppingItemInlineForm


__all__ = [
    "CategoryAdminForm",
    "GlobalSearchForm",
    "ProductAdminForm",
    "ShoppingItemInlineForm",
]
